/*
 * Copyright (c) 2007-2008 University of Washington
 * 
 * All rights reserved. For information, contact the Adeona research
 * team at adeona@cs.washington.edu
 *
 * See the file License.txt included in this distribution for details.
 *
 */

#ifndef _STATE_H_
#define _STATE_H_

#include <time.h>

#include "common.h"
#include "crypto.h"
#include "ctxtcache.h"
#include "opendht.h"
#include "options.h"


struct state_data
{
    prgkey_t nextseed;  
    prgkey_t nextcacheseed;  
    prgblock_t schedseed; //seed for delta time
    renckey_t renckey;
    dhtkey_t* dhtkeys; //dht keys to insert to
};

typedef struct _state
{
    //saved state
    time_t updatetime;
    prgkey_t currentseed;
    prgkey_t cacheseed;

    //ciphertext cache, saved in client state
    cache_elem_t* cache;

    //helper state, not saved
    struct state_data* sd;
} state_t;

/*
 * Indices for the different components of state_t.
 * These are for the calls to the PRG generator
 *
 */ 
#define STATE_NEXTSEED_LOW     0
#define STATE_NEXTSEED_HIGH    sizeof( prgkey_t ) - 1

#define STATE_NEXTCACHESEED_LOW       STATE_NEXTSEED_HIGH + 1
#define STATE_NEXTCACHESEED_HIGH      STATE_NEXTCACHESEED_LOW + sizeof(prgkey_t) - 1

#define STATE_DSEED_LOW       STATE_NEXTCACHESEED_HIGH + 1    
#define STATE_DSEED_HIGH      STATE_DSEED_LOW + sizeof(prgblock_t) - 1

//#define STATE_ENC_LOW         STATE_DSEED_HIGH + 1
//#define STATE_ENC_HIGH        STATE_ENC_LOW + PRG_KEYSIZE - 1

//#define STATE_HMAC_LOW        STATE_ENC_HIGH + 1
//#define STATE_HMAC_HIGH       STATE_HMAC_LOW + PRG_KEYSIZE - 1

#define STATE_RENC_LOW       STATE_DSEED_HIGH + 1
#define STATE_RENC_HIGH      STATE_RENC_LOW + sizeof( renckey_t ) - 1

#define STATE_DHT_LOW         STATE_RENC_HIGH + 1
#define STATE_DHT_HIGH        STATE_DHT_LOW + sizeof( dhtkey_t )*opts.num_dhtkeys_totry - 1

/* 
 * Flags for operations on different components of state_t 
 *
 */
#define SC_SAVED    ( 0x01 << 0 )
#define SC_CACHE    ( 0x01 << 1 )
#define SC_DATA     ( 0x01 << 2 )
#define SC_NOCACHE  SC_SAVED + SC_DATA
#define SC_ALL      SC_SAVED + SC_CACHE + SC_DATA

/*
 * Sizes of the difference components of state_t
 *
 */
#define SC_SAVED_SIZE ( sizeof( time_t ) + ( sizeof( prgkey_t ) * 2 ) )


state_t* state_alloc( void );
int state_save_filename( char* filename IN, state_t* state IN, char* pwdstr IN, int plen IN, uint flags IN);
int state_save( FILE* file IN, state_t* state IN, char* pwdstr IN, int plen IN, uint flags IN);
int state_load( FILE* statefile IN, state_t* state INOUT, char* pwdstr IN, int plen IN, uint flags IN );
int state_load_filename( char* filename IN, state_t* state INOUT, char* pwdstr IN, int plen IN, uint flags IN );
int state_makenew( state_t* state INOUT );
int state_copy( state_t* dst INOUT, state_t* src IN, uint flags IN );
void state_free( state_t* state IN );

#endif // _STATE_H_

