/*
 * Copyright (c) 2007-2008 University of Washington
 * 
 * All rights reserved. For information, contact the Adeona research
 * team at adeona@cs.washington.edu
 *
 * See the file License.txt included in this distribution for details.
 *
 */

#include "encrypt.h"
#include "log.h"

/* 
 * Encrypt with AES in CTR mode by using prg_generate to generate the
 * keystream.
 *
 * Supports in-place encryption i.e. <pt> == <ct>
 *
 * IMPORTANT: fixed IV so <key> should be one-time
 *
 */ 
int encryption( enckey_t key IN, uchar* pt IN, uchar* ct OUT, int len IN )
{
    if( key != NULL && pt != NULL && ct != NULL && len > 0 ) {
        uchar stream[len];
        memset( stream, 0x00, len );
        
        if( prg_generate( key, 0, len-1, stream ) == FAILURE ) {
            error( "could not generate keystream" );
            return FAILURE;
        }
        
        int i;
        for( i=0; i<len; i++ ) {
            ct[i] = pt[i] ^ stream[i];
        }
        
        return SUCCESS;
    }
    else {
        error( "could not generate keystream" );
        return FAILURE;
    }

}

int decryption( enckey_t key IN, uchar* ct IN, uchar* pt OUT, int len IN )
{
    return encryption( key, ct, pt, len );
}
