/*
 * Copyright (c) 2007-2008 University of Washington
 * 
 * All rights reserved. For information, contact the Adeona research
 * team at adeona@cs.washington.edu
 *
 * See the file License.txt included in this distribution for details.
 *
 */
#ifndef _LOCATION_H_
#define _LOCATION_H_

#include <time.h>

#include "common.h"
#include "ipaddress.h"
#include "traceroute.h"
#include "landmarks.h"
#include "webcam.h"
#include "accesspoint.h"

#define LOC_DATA_UPPERBOUND \
   ( sizeof( time_t ) +  ( 2*sizeof( struct in_addr ) ) + ( sizeof( apdata_t ) ) \
     + TR_DATA_UPPERBOUND + WEBCAM_IMGDATA_UPPERBOUND )

typedef struct _location {
    time_t         timestamp;
    struct in_addr intIP;
    struct in_addr extIP;
    apdata_t ap;
    uchar*         traceroutes;
    webcamimg_t    webcamimg; 
} location_t;

int location_alloc( location_t* loc );
int location_copy( location_t* dest, location_t* src );
int location_serialize( location_t* loc IN, uchar* buf OUT, int len IN );
int location_deserialize( uchar* buf IN, int len IN, location_t* loc OUT );
int location_free( location_t* loc ); 

int location_get_data( location_t* loc OUT );
int location_checkchanged();
int location_compare( location_t* loc1, location_t* loc2 );
int location_save_data( char* filename IN, location_t* loc IN );

void location_printdata( FILE* file, void* ptr IN );
void location_printdata_compact( FILE* file, void* ptr IN );
void location_printdata_friendly( FILE* file, void* ptr IN );

#endif // _LOCATION_H_
