/*
 * Copyright (c) 2007-2008 University of Washington
 * 
 * All rights reserved. For information, contact the Adeona research
 * team at adeona@cs.washington.edu
 *
 * See the file License.txt included in this distribution for details.
 *
 */

#include <arpa/inet.h>
#include "common.h"
#include "traceroute.h"
#include "ipaddress.h"
#include "location.h"
#include "webcam.h"
#include "util.h"
#include "log.h"
#include "accesspoint.h"
#include "options.h"

static void test_getip( void )
{
    struct in_addr ipint;
    struct in_addr ipext;
    memset( &ipint, 0x00, sizeof( struct in_addr ) );
    memset( &ipext, 0x00, sizeof( struct in_addr ) );
    assert( memcmp( "0.0.0.0", inet_ntoa( ipint ), 7 ) == 0 );
    assert( memcmp( "0.0.0.0", inet_ntoa( ipext ), 7 ) == 0 );
    
    get_internalip( &ipint );
    printout( "internal ip: %s\n", inet_ntoa( ipint ) );

    get_externalip( &ipext );
    printout( "external ip: %s\n", inet_ntoa( ipext ) );
}

static void test_traceroute( void )
{
    uchar* trdata = NULL;
    int len = 0;
    
    landmarks_load( "resources/landmarks.adeona" );
    
    get_traceroutes( &trdata, &len );
    print_traceroutes( stdout, trdata, len, NORMAL );
    printout( "\n" );
    print_traceroutes( stdout, trdata, len, COMPACT );
    printout( "\n" );
    print_traceroutes( stdout, trdata, len, FRIENDLY );
    
    landmarks_cleanup();
    free( trdata );
}

static void test_getdata( void )
{
    location_t loc;

    location_alloc( &loc );
    landmarks_load( "resources/landmarks.adeona" );
    assert( landmarks_isloaded() == TRUE );

    location_get_data( &loc );
    location_printdata( stdout, &loc );
    
    landmarks_cleanup();
    assert( landmarks_isloaded() == FALSE );

    location_free( &loc );
}

static void get_webcam_data( void )
{
    webcamimg_t img;

    assert( webcam_snapshot_get( &img ) == SUCCESS );
    assert( webcam_snapshot_save( "testimage.jpg", &img ) == SUCCESS );
    
    free( img.image );
}

static void get_ssid_data( void )
{
    apdata_t apd;
    memset( &apd, 0x00, sizeof( apd ) );

    ap_ssid_get( &apd );
    
    printout( "X.%s.X\n", apd.ssid );
}

int main( void )
{
    load_default_options();
    opts.have_console = TRUE;
//    write_traceroutes();  
//    load_traceroutes();

    test_getip();
    test_traceroute();
//    test_getdata();
//    get_webcam_data();
    get_ssid_data();

    exit( EXIT_SUCCESS );
}


#if 0 
//DEPRECATED TESTS

/*
 * This writes out <buffer> to a file so we can read it from a
 * platform with different endian-ness and test if get_location_data is
 * doing endian stuff correctly
 *
 */ 
static void write_traceroutes( void )
{
    uchar buffer[LOC_DATA_UPPERBOUND];
    memset( buffer, 0x00, LOC_DATA_UPPERBOUND );
    location_init();
    get_location_data( buffer, sizeof( buffer ) );
    location_cleanup();

    location_printdata( stdout, buffer, sizeof( buffer ) );
    
    FILE* file = fopen( "traceroutes_test.txt", "w" );
    assert( file != NULL );
    
    fprintln_bin2hex( file, buffer, sizeof( buffer ) );
    
    fclose( file );
}

//#define TR_FILENAME "traceroutes_test_from_x86.txt"
//#define TR_FILENAME "traceroutes_test_from_powerpc.txt"
#define TR_FILENAME "buggy_print_location_data.txt"
static void load_traceroutes( void )
{
    FILE* file = fopen( TR_FILENAME, "r" );
    assert( file != NULL );
    
    uchar buffer[LOC_DATA_UPPERBOUND];
    memset( buffer, 0x00, LOC_DATA_UPPERBOUND );
    freadln_hex2bin( file, buffer, sizeof( buffer ) );
    
    location_printdata( stdout, buffer, sizeof( buffer ) );
    
    fclose( file );
}
#endif //END DEPRECATED TESTS

