/*
 * Copyright (c) 2007-2008 University of Washington
 * 
 * All rights reserved. For information, contact the Adeona research
 * team at adeona@cs.washington.edu
 *
 * See the file License.txt included in this distribution for details.
 *
 */
#include "common.h"
#include "options.h"
#include "remstore.h"
#include "util.h"
#include "log.h"
#include <unistd.h>
#include "ctxtcache.h"

static void test_putget( void )
{
    if( remstore_connect() == FAILURE ) {
        debug( "can't connect" );
    } 

    dhtkey_t randomkey;
    get_random_bytes( randomkey, sizeof( dhtkey_t ) );
    
    log_msg_bin( ADEONA_DEBUG, randomkey, sizeof( dhtkey_t ), "using key:" );

    int bdlen = 1024 * 20 + 1;
    uchar bigdata[bdlen];
    get_random_bytes( bigdata, bdlen ); 
    
    if( remstore_put( randomkey, bigdata, bdlen ) == FAILURE ) {
        debug( "cannot put" );
    }   
    
    uchar* bigget = NULL;
    int bglen = 0;
    getiter_t iter;
    
    if( remstore_getiter_init( randomkey, &iter ) == FAILURE ) {
        debug( "cannot init get" );
    }
    
    if( remstore_getiter_next_alloc( &iter, &bigget, &bglen ) == FAILURE ) {
        debug( "cannot get" );
    }
    assert( bigget != NULL && bglen > 0 );
    
    remstore_getiter_done( &iter );
    remstore_disconnect();
    
    debug( "putlen = %d, getlen = %d", bdlen, bglen );
    assert( bdlen == bglen );
    assert( memcmp( bigdata, bigget, bdlen ) == 0 );
    free( bigget );

    printout( "***** %s passed\n", __FUNCTION__ );

}

static void do_get( void )
{
    int i;
    uchar* hexkey= "3BC79B4F92E8BD49367755F0F2C5A9C2F32393E2";
    dhtkey_t key;
    hex2bin( hexkey, strlen( hexkey ), key, sizeof( dhtkey_t ) );
    
    uchar* get = NULL;
    int glen = 0;
    getiter_t iter;

    remstore_connect();
    remstore_getiter_init( key, &iter );
    remstore_getiter_next_alloc( &iter, &get, &glen );
    remstore_getiter_done( &iter );
    remstore_disconnect();
    
    if( get != NULL ) {
        println_bin2hex( get, glen );
    }

}

static void do_putget( void )
{
    debug( "==================== new test ====================" );
    assert( gateways_isloaded() == TRUE );
    if( remstore_connect() == FAILURE ) {
        debug( "can't connect" );
    } 

    dhtkey_t randomkey;
    get_random_bytes( randomkey, sizeof( dhtkey_t ) );
    
    log_msg_bin( ADEONA_DEBUG, randomkey, sizeof( dhtkey_t ), "using key:" );

//    int bdlen = 1024 * 20 + 1;
    int bdlen = CTXT_QUEUE_LEN;
    uchar bigdata[bdlen];
    get_random_bytes( bigdata, bdlen ); 
    
    debug( "========== doing put" );
    if( remstore_put( randomkey, bigdata, bdlen ) == FAILURE ) {
        debug( "========== cannot put" );
    }
    else {
        debug( "========== put successful" );
    }
    remstore_disconnect();
    debug( "" );
    sleep( 2 );
    if( remstore_connect() == FAILURE ) {
        debug( "can't connect" );
    } 

    uchar* bigget = NULL;
    int bglen = 0;
    getiter_t iter;
    
    debug( "========== doing get" );
    if( remstore_getiter_init( randomkey, &iter ) == FAILURE ) {
        debug( "========== cannot get" );
    }
    else {
        if( remstore_getiter_next_alloc( &iter, &bigget, &bglen ) == FAILURE ) {
            debug( "========== cannot parse get data" );
        }
        
        remstore_getiter_done( &iter );
    }
    
    remstore_disconnect();
    
    if( bigget != NULL && bglen > 0 ) {
        debug( "putlen = %d, getlen = %d", bdlen, bglen );
        if( memcmp( bigdata, bigget, bdlen ) == 0 ) {
            debug( "========== get successful" );
        }
        else {
            debug( "========== get failure" );
        }

        free( bigget );
    }
    
    gateways_free();
    assert( gateways_isloaded() == FALSE );
    debug( "==================== test done ===================" );
}


int main( void )
{
    setup_resources( "resources/", RES_ALL );
    do_putget();
//    do_get();
//    test_putget();

    exit( EXIT_SUCCESS );
}
